=== modified file 'NEWS'
--- NEWS	2008-06-06 12:02:48 +0000
+++ NEWS	2008-06-06 19:07:50 +0000
@@ -31,6 +31,9 @@
       ``modified`` (for null-separated unknowns, use ``ls --unknown --null``)
       (Adrian Wilkins)
 
+    * Faster branching (1.09x) and lightweight checkouts (1.06x) on large trees.
+      (Ian Clatworthy, Aaron Bentley)
+
   BUGFIXES:
 
     * ``bzr status`` was breaking if you merged the same revision twice.

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2008-05-12 02:48:08 +0000
+++ bzrlib/tests/test_transform.py	2008-06-06 16:36:15 +0000
@@ -64,7 +64,7 @@
 
     def get_transform(self):
         transform = TreeTransform(self.wt)
-        #self.addCleanup(transform.finalize)
+        self.addCleanup(transform.finalize)
         return transform, transform.root
 
     def test_existing_limbo(self):
@@ -1704,6 +1704,19 @@
         self.assertEqual([], list(target.iter_changes(revision_tree)))
         self.assertTrue(source.is_executable('file1-id'))
 
+    def test_case_insensitive_build_tree_inventory(self):
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/file', 'source/FILE'])
+        source.add(['file', 'FILE'], ['lower-id', 'upper-id'])
+        source.commit('added files')
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case insensitive
+        target = self.make_branch_and_tree('target')
+        target.case_sensitive = False
+        build_tree(source.basis_tree(), target, source, delta_from_tree=True)
+        self.assertEqual('file.moved', target.id2path('lower-id'))
+        self.assertEqual('FILE', target.id2path('upper-id'))
+
 
 class MockTransform(object):
 

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2008-05-12 02:48:08 +0000
+++ bzrlib/transform.py	2008-06-07 19:33:26 +0000
@@ -450,16 +450,21 @@
         del self._new_id[trans_id]
         del self._r_new_id[file_id]
 
-    def new_paths(self):
-        """Determine the paths of all new and changed files"""
+    def new_paths(self, filesystem_only=False):
+        """Determine the paths of all new and changed files.
+
+        :param filesystem_only: if True, only calculate values for files
+            that require renames or execute bit changes.
+        """
         new_ids = set()
-        fp = FinalPaths(self)
-        for id_set in (self._new_name, self._new_parent, self._new_contents,
-                       self._new_id, self._new_executability):
+        if filesystem_only:
+            id_sets = (self._needs_rename, self._new_executability)
+        else:
+            id_sets = (self._new_name, self._new_parent, self._new_contents,
+                       self._new_id, self._new_executability)
+        for id_set in id_sets:
             new_ids.update(id_set)
-        new_paths = [(fp.get_path(t), t) for t in new_ids]
-        new_paths.sort()
-        return new_paths
+        return sorted(FinalPaths(self).get_paths(new_ids))
 
     def tree_kind(self, trans_id):
         """Determine the file kind in the working tree.
@@ -856,7 +861,8 @@
     def _set_executability(self, path, entry, trans_id):
         """Set the executability of versioned files """
         new_executability = self._new_executability[trans_id]
-        entry.executable = new_executability
+        if entry is not None:
+            entry.executable = new_executability
         if supports_executable():
             abspath = self._tree.abspath(path)
             current_mode = os.stat(abspath).st_mode
@@ -1167,7 +1173,7 @@
                                    tree.case_sensitive)
         self._deletiondir = deletiondir
 
-    def apply(self, no_conflicts=False, _mover=None):
+    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
         """Apply all changes to the inventory and filesystem.
 
         If filesystem or inventory conflicts are present, MalformedTransform
@@ -1177,13 +1183,20 @@
 
         :param no_conflicts: if True, the caller guarantees there are no
             conflicts, so no check is made.
+        :param precomputed_delta: An inventory delta to use instead of
+            calculating one.
         :param _mover: Supply an alternate FileMover, for testing
         """
         if not no_conflicts:
             conflicts = self.find_conflicts()
             if len(conflicts) != 0:
                 raise MalformedTransform(conflicts=conflicts)
-        inventory_delta = []
+        if precomputed_delta is None:
+            new_inventory_delta = []
+            inventory_delta = new_inventory_delta
+        else:
+            new_inventory_delta = None
+            inventory_delta = precomputed_delta
         child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
             if _mover is None:
@@ -1192,9 +1205,10 @@
                 mover = _mover
             try:
                 child_pb.update('Apply phase', 0, 2)
-                self._apply_removals(inventory_delta, mover)
+                self._apply_removals(new_inventory_delta, mover)
                 child_pb.update('Apply phase', 1, 2)
-                modified_paths = self._apply_insertions(inventory_delta, mover)
+                modified_paths = self._apply_insertions(new_inventory_delta,
+                                                        mover)
             except:
                 mover.rollback()
                 raise
@@ -1213,6 +1227,8 @@
         That is, delete files that are to be deleted, and put any files that
         need renaming into limbo.  This must be done in strict child-to-parent
         order.
+
+        If inventory_delta is None, no inventory delta generation is performed.
         """
         tree_paths = list(self._tree_path_ids.iteritems())
         tree_paths.sort(reverse=True)
@@ -1234,7 +1250,8 @@
                             raise
                     else:
                         self.rename_count += 1
-                if trans_id in self._removed_id:
+                if (trans_id in self._removed_id
+                    and inventory_delta is not None):
                     if trans_id == self._new_root:
                         file_id = self._tree.get_root_id()
                     else:
@@ -1252,62 +1269,78 @@
         That is, create any files that need to be created, and restore from
         limbo any files that needed renaming.  This must be done in strict
         parent-to-child order.
+
+        If inventory_delta is None, no inventory delta is calculated, and
+        no list of modified paths is returned.
         """
-        new_paths = self.new_paths()
+        new_paths = self.new_paths(filesystem_only=(inventory_delta is None))
         modified_paths = []
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         completed_new = []
+        new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
+                                 new_paths)
+        if inventory_delta is not None:
+            entries = self._tree.iter_entries_by_dir(
+                new_path_file_ids.values())
+            old_paths = dict((e.file_id, p) for p, e in entries)
+        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
             for num, (path, trans_id) in enumerate(new_paths):
                 new_entry = None
-                child_pb.update('adding file', num, len(new_paths))
-                if trans_id in self._new_contents or \
-                    self.path_changed(trans_id):
-                    full_path = self._tree.abspath(path)
-                    if trans_id in self._needs_rename:
+                if (num % 10) == 0:
+                    child_pb.update('adding file', num, len(new_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._needs_rename:
+                    try:
+                        mover.rename(self._limbo_name(trans_id), full_path)
+                    except OSError, e:
+                        # We may be renaming a dangling inventory id
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        self.rename_count += 1
+                if inventory_delta is not None:
+                    if (trans_id in self._new_contents or
+                        self.path_changed(trans_id)):
+                        if trans_id in self._new_contents:
+                            modified_paths.append(full_path)
+                            completed_new.append(trans_id)
+                    file_id = new_path_file_ids[trans_id]
+                    if file_id is not None and (trans_id in self._new_id or
+                        trans_id in self._new_name or
+                        trans_id in self._new_parent
+                        or trans_id in self._new_executability):
                         try:
-                            mover.rename(self._limbo_name(trans_id), full_path)
-                        except OSError, e:
-                            # We may be renaming a dangling inventory id
-                            if e.errno != errno.ENOENT:
-                                raise
+                            kind = self.final_kind(trans_id)
+                        except NoSuchFile:
+                            kind = self._tree.stored_kind(file_id)
+                        parent_trans_id = self.final_parent(trans_id)
+                        parent_file_id = new_path_file_ids.get(parent_trans_id)
+                        if parent_file_id is None:
+                            parent_file_id = self.final_file_id(
+                                parent_trans_id)
+                        if trans_id in self._new_reference_revision:
+                            new_entry = inventory.TreeReference(
+                                file_id,
+                                self._new_name[trans_id],
+                                self.final_file_id(self._new_parent[trans_id]),
+                                None, self._new_reference_revision[trans_id])
                         else:
-                            self.rename_count += 1
-                    if trans_id in self._new_contents:
-                        modified_paths.append(full_path)
-                        completed_new.append(trans_id)
-                file_id = self.final_file_id(trans_id)
-                if file_id is not None and (trans_id in self._new_id or
-                    trans_id in self._new_name or trans_id in self._new_parent
-                    or trans_id in self._new_executability):
-                    try:
-                        kind = self.final_kind(trans_id)
-                    except NoSuchFile:
-                        kind = self._tree.stored_kind(file_id)
-                    if trans_id in self._new_reference_revision:
-                        new_entry = inventory.TreeReference(
-                            self.final_file_id(trans_id),
-                            self._new_name[trans_id],
-                            self.final_file_id(self._new_parent[trans_id]),
-                            None, self._new_reference_revision[trans_id])
-                    else:
-                        new_entry = inventory.make_entry(kind,
-                            self.final_name(trans_id),
-                            self.final_file_id(self.final_parent(trans_id)),
-                            self.final_file_id(trans_id))
-                    try:
-                        old_path = self._tree.id2path(new_entry.file_id)
-                    except errors.NoSuchId:
-                        old_path = None
-                    inventory_delta.append((old_path, path, new_entry.file_id,
-                                            new_entry))
+                            new_entry = inventory.make_entry(kind,
+                                self.final_name(trans_id),
+                                parent_file_id, file_id)
+                        old_path = old_paths.get(new_entry.file_id)
+                        inventory_delta.append(
+                            (old_path, path, new_entry.file_id, new_entry))
 
                 if trans_id in self._new_executability:
                     self._set_executability(path, new_entry, trans_id)
         finally:
             child_pb.finished()
-        for trans_id in completed_new:
-            del self._new_contents[trans_id]
+        if inventory_delta is None:
+            self._new_contents.clear()
+        else:
+            for trans_id in completed_new:
+                del self._new_contents[trans_id]
         return modified_paths
 
 
@@ -1554,6 +1587,10 @@
             self._known_paths[trans_id] = self._determine_path(trans_id)
         return self._known_paths[trans_id]
 
+    def get_paths(self, trans_ids):
+        return [(self.get_path(t), t) for t in trans_ids]
+
+
 
 def topology_sorted_ids(tree):
     """Determine the topological order of the ids in a tree"""
@@ -1562,7 +1599,8 @@
     return file_ids
 
 
-def build_tree(tree, wt, accelerator_tree=None, hardlink=False):
+def build_tree(tree, wt, accelerator_tree=None, hardlink=False,
+               delta_from_tree=False):
     """Create working tree for a branch, using a TreeTransform.
     
     This function should be used on empty trees, having a tree root at most.
@@ -1584,6 +1622,8 @@
     :param hardlink: If true, hard-link files to accelerator_tree, where
         possible.  accelerator_tree must implement abspath, i.e. be a
         working tree.
+    :param delta_from_tree: If true, build_tree may use the input Tree to
+        generate the inventory delta.
     """
     wt.lock_tree_write()
     try:
@@ -1592,7 +1632,8 @@
             if accelerator_tree is not None:
                 accelerator_tree.lock_read()
             try:
-                return _build_tree(tree, wt, accelerator_tree, hardlink)
+                return _build_tree(tree, wt, accelerator_tree, hardlink,
+                                   delta_from_tree)
             finally:
                 if accelerator_tree is not None:
                     accelerator_tree.unlock()
@@ -1602,11 +1643,14 @@
         wt.unlock()
 
 
-def _build_tree(tree, wt, accelerator_tree, hardlink):
+def _build_tree(tree, wt, accelerator_tree, hardlink, delta_from_tree):
     """See build_tree."""
     for num, _unused in enumerate(wt.all_file_ids()):
         if num > 0:  # more than just a root
             raise errors.WorkingTreeAlreadyPopulated(base=wt.basedir)
+    existing_files = set()
+    for dir, files in wt.walkdirs():
+        existing_files.update(f[0] for f in files)
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
@@ -1631,6 +1675,10 @@
         try:
             deferred_contents = []
             num = 0
+            if delta_from_tree:
+                precomputed_delta = []
+            else:
+                precomputed_delta = None
             for num, (tree_path, entry) in \
                 enumerate(tree.inventory.iter_entries_by_dir()):
                 pb.update("Building tree", num - len(deferred_contents),
@@ -1639,12 +1687,11 @@
                     continue
                 reparent = False
                 file_id = entry.file_id
-                target_path = wt.abspath(tree_path)
-                try:
+                if delta_from_tree:
+                    precomputed_delta.append((None, tree_path, file_id, entry))
+                if tree_path in existing_files:
+                    target_path = wt.abspath(tree_path)
                     kind = file_kind(target_path)
-                except NoSuchFile:
-                    pass
-                else:
                     if kind == "directory":
                         try:
                             bzrdir.BzrDir.open(target_path)
@@ -1670,7 +1717,7 @@
                     file_trans_id[file_id] = trans_id
                     tt.version_file(entry.file_id, trans_id)
                     executable = tree.is_executable(entry.file_id, tree_path)
-                    if executable is not None:
+                    if executable:
                         tt.set_executability(executable, trans_id)
                     deferred_contents.append((entry.file_id, trans_id))
                 else:
@@ -1689,6 +1736,8 @@
         divert_trans = set(file_trans_id[f] for f in divert)
         resolver = lambda t, c: resolve_checkout(t, c, divert_trans)
         raw_conflicts = resolve_conflicts(tt, pass_func=resolver)
+        if len(raw_conflicts) > 0:
+            precomputed_delta = None
         conflicts = cook_conflicts(raw_conflicts, tt)
         for conflict in conflicts:
             warning(conflict)
@@ -1696,7 +1745,8 @@
             wt.add_conflicts(conflicts)
         except errors.UnsupportedOperation:
             pass
-        result = tt.apply(no_conflicts=True)
+        result = tt.apply(no_conflicts=True,
+                          precomputed_delta=precomputed_delta)
     finally:
         tt.finalize()
         top_pb.finished()

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-06-04 17:54:52 +0000
+++ bzrlib/workingtree_4.py	2008-06-06 16:40:46 +0000
@@ -1373,8 +1373,11 @@
                 if basis_root_id is not None:
                     wt._set_root_id(basis_root_id)
                     wt.flush()
+                # delta_from_tree is safe even for DirStateRevisionTrees,
+                # because wt4.apply_inventory_delta does not mutate the input
+                # inventory entries.
                 transform.build_tree(basis, wt, accelerator_tree,
-                                     hardlink=hardlink)
+                                     hardlink=hardlink, delta_from_tree=True)
             finally:
                 basis.unlock()
         finally:

